// AOTSystem.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include "AOTSystem.h"

// global data within shared data segment to allow sharing across instances

#pragma data_seg(".SHARDATA")

HWND g_hwndMain = NULL;
HHOOK g_hookShell = NULL;
HHOOK g_hookMenu = NULL;
HINSTANCE g_hInstance = NULL;

UINT AOT_AOT;

#pragma data_seg()
#pragma comment(linker, "/section:.SHARDATA,rws")

// extern "C" prevents name mangling so that procedures can be referenced
// from outside the DLL

extern "C"
{
	static LRESULT CALLBACK ShellProc(int nCode, WPARAM wParam, LPARAM lParam);
	static LRESULT CALLBACK MenuProc(int nCode, WPARAM wParam, LPARAM lParam);
	static BOOL CALLBACK AddMonitorsCallBack(HMONITOR hMonitor, // handle to display monitor
	HDC hdcMonitor, // handle to monitor DC
	LPRECT lprcMonitor, // monitor intersection rectangle
	LPARAM dwData   	// data
	);
}

// the entry point for the DLL

BOOL APIENTRY DllMain(HINSTANCE hInstance, DWORD  dwReason, LPVOID lpReserved)
{
	switch (dwReason)
	{
	case DLL_PROCESS_ATTACH:
		// first time DLL is loaded (hopefully by our main AOT)

		g_hInstance = hInstance;

		AOT_AOT = ::RegisterWindowMessage(AOT_AOT_MSG);

		m_MonitorArray.SetSize(::GetSystemMetrics(SM_CMONITORS));
		ADDMONITOR addMonitor;
		addMonitor.pMonitors = &m_MonitorArray;
		addMonitor.currentIndex = 0;
		::EnumDisplayMonitors(NULL, NULL, AddMonitorsCallBack, (LPARAM)&addMonitor);

		return TRUE;			// successfully loaded DLL
	}

	return TRUE;				// successfully did ... whatever
}


static BOOL CALLBACK AddMonitorsCallBack(HMONITOR hMonitor, HDC hdcMonitor, LPRECT lprcMonitor, LPARAM dwData)
{
	LPADDMONITOR pAddMonitor = (LPADDMONITOR)dwData;

	CMonitor* pMonitor = new CMonitor;
	pMonitor->Attach(hMonitor);

	pAddMonitor->pMonitors->SetAt(pAddMonitor->currentIndex, pMonitor);
	pAddMonitor->currentIndex++;
	CString tmp;
	CRect monitorRect;
	pMonitor->GetMonitorRect(&monitorRect);
	tmp.Format("%s%d", AOT_DISPLAY_MSG, pAddMonitor->currentIndex);
	pMonitor->AOT_DISP = ::RegisterWindowMessage(tmp);

	return TRUE;
}

__declspec(dllexport) BOOL SetHook(HWND hWnd)
{
	if (g_hwndMain != NULL)
		return FALSE;			// we already have a main window, so hooks must already be installed

	// shell hook to add to context menu
	g_hookShell = SetWindowsHookEx(WH_CALLWNDPROC, (HOOKPROC)ShellProc, g_hInstance, 0);

	// menu hook to handle the menu message
	g_hookMenu = SetWindowsHookEx(WH_GETMESSAGE, (HOOKPROC)MenuProc, g_hInstance, 0);

	if ((g_hookShell != NULL) && (g_hookMenu != NULL))
	{
		g_hwndMain = hWnd;		// the main window will be whichever window called SetHook
		// ie our main dialog window

		return TRUE;			// both hooks were successfully installed
	}

	return FALSE;				// one or both of the hooks was not installed successfully
}

__declspec(dllexport) BOOL ClearHook(HWND hWnd)
{
	if (hWnd != g_hwndMain || hWnd == NULL)
		return FALSE;

	BOOL unhooked = UnhookWindowsHookEx(g_hookShell);
	if (!unhooked)
		return FALSE;			// did not successfully uninstall the shell hook

	unhooked = UnhookWindowsHookEx(g_hookMenu);
	if (!unhooked)
		return FALSE;			// did not successfully uninstall the menu hook

	g_hwndMain = NULL;			// successfully uninstalled both hooks therefore do not need
	// to liaise with main window any longer
	return unhooked;
}

// ShellProc - adds / removes the "Toujours visible" entry from the system menu

static LRESULT CALLBACK ShellProc(int nCode, WPARAM wParam, LPARAM lParam)
{
	if (nCode < 0)
	{
		return CallNextHookEx(g_hookShell, nCode, wParam, lParam);
	}

	CWPSTRUCT* wps = (CWPSTRUCT*)lParam;
	HWND hWnd = (HWND)(wps->hwnd);

	if (wps->message == WM_MENUSELECT)
	{
		if (wps->lParam == NULL && HIWORD(wps->wParam) == 0xFFFF)
		{
			RemoveMenu(GetSystemMenu(hWnd, FALSE), AOT_AOT, MF_BYCOMMAND);
			for (int i = 0; i < m_MonitorArray.GetSize(); i++)
			{
				CMonitor* monitor = (CMonitor*)m_MonitorArray.GetAt(i);
				RemoveMenu(GetSystemMenu(hWnd, FALSE), monitor->AOT_DISP, MF_BYCOMMAND);
			}
		}
	}

	if (wps->message == WM_INITMENUPOPUP)
	{
		HMENU hMenu = (HMENU)wps->wParam;
		if ((IsMenu(hMenu) & (HIWORD(wps->lParam) == TRUE)))
		{
			if (GetWindowLong(hWnd, GWL_EXSTYLE) & WS_EX_TOPMOST)
			{
				AppendMenu(GetSystemMenu(hWnd, FALSE), MF_CHECKED | MF_BYPOSITION | MF_STRING, AOT_AOT, "Toujours visible");
			}
			else
			{
				AppendMenu(GetSystemMenu(hWnd, FALSE), MF_BYPOSITION | MF_STRING, AOT_AOT, "Toujours visible");
			}
			CRect windowRect;
			::GetWindowRect(hWnd, &windowRect);

			WINDOWPLACEMENT wPlacement;
			::GetWindowPlacement(HWND(hWnd), &wPlacement);

			for (int i = 0; i < m_MonitorArray.GetSize(); i++)
			{
				CMonitor* monitor = (CMonitor*)m_MonitorArray.GetAt(i);
				CString tmp, tmp1;
				tmp.Format("Envoyer vers moniteur %d", i + 1);				
				CRect winRect;
				CWnd::FromHandle(hWnd)->GetWindowRect( &winRect );

				if (!monitor->IsOnMonitor(CWnd::FromHandle(hWnd))||!monitor->IsOnMonitor(winRect.CenterPoint()))
				{
//					if (wPlacement.showCmd != SW_MINIMIZE && wPlacement.showCmd != SW_HIDE&&wPlacement.showCmd != SW_SHOWMINIMIZED)
//						if (wPlacement.showCmd != SW_MAXIMIZE || (wPlacement.showCmd == SW_MAXIMIZE && wPlacement.ptMinPosition.x != monitorRect.left))
							AppendMenu(GetSystemMenu(hWnd, FALSE), MF_BYPOSITION | MF_STRING, monitor->AOT_DISP, tmp);
				}
			}
		}
	}

	return CallNextHookEx(g_hookShell, nCode, wParam, lParam);
}

static LRESULT CALLBACK MenuProc(int nCode, WPARAM wParam, LPARAM lParam)
{
	if (nCode < 0)
	{
		return CallNextHookEx(g_hookMenu, nCode, wParam, lParam);
	}

	MSG* msg = (MSG*)lParam;

	if(msg->message == WM_SYSCOMMAND)
	{
		if(LOWORD(msg->wParam) == AOT_AOT)
		{
			if (GetWindowLong(HWND(msg->hwnd), GWL_EXSTYLE) & WS_EX_TOPMOST)
			{
				SetWindowPos(HWND(msg->hwnd), HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_FRAMECHANGED);
			}
			else
			{
				SetWindowPos(HWND(msg->hwnd), HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_FRAMECHANGED);
			}
		}
		for (int i = 0; i < m_MonitorArray.GetSize(); i++)
		{
			CMonitor* monitor = (CMonitor*)m_MonitorArray.GetAt(i);
			if (LOWORD(msg->wParam) == monitor->AOT_DISP)
			{
				WINDOWPLACEMENT wPlacement;
				UINT oldShowCmd;
				CRect monitorRect;
				monitor->GetMonitorRect(monitorRect);

				::GetWindowPlacement(HWND(msg->hwnd), &wPlacement);
			
				oldShowCmd = wPlacement.showCmd;
				if(oldShowCmd == SW_SHOWMAXIMIZED) {
					wPlacement.showCmd = SW_RESTORE;
					::SetWindowPlacement(HWND(msg->hwnd), &wPlacement);
				}
				if(oldShowCmd != SW_SHOWMINIMIZED) {
					monitor->CenterWindowToMonitor(CWnd::FromHandle(msg->hwnd));

				}
				if(oldShowCmd == SW_SHOWMAXIMIZED) {
					::GetWindowPlacement(HWND(msg->hwnd), &wPlacement);
					wPlacement.showCmd = SW_SHOWMAXIMIZED;
					wPlacement.ptMaxPosition = monitorRect.TopLeft();
					::SetWindowPlacement(HWND(msg->hwnd), &wPlacement);
				}
/*				CRect monitorRect;
				monitor->GetMonitorRect(monitorRect);
				CRect windowRect;
				CString tmp;
				tmp.Format("%d: %d %d %d %d", i, wPlacement.ptMinPosition.x, wPlacement.ptMinPosition.y, wPlacement.ptMaxPosition.x, wPlacement.ptMaxPosition.y);
				//AfxMessageBox(tmp,IDOK);

				tmp.Format("%d %d %d %d", wPlacement.ptMinPosition.x, wPlacement.ptMinPosition.y, wPlacement.ptMaxPosition.x, wPlacement.ptMaxPosition.y);
				//AfxMessageBox(tmp,IDOK);


				::GetWindowRect(HWND(msg->hwnd), &windowRect);
				CRect newRect(monitorRect.left, monitorRect.top, monitorRect.left + windowRect.Width(), monitorRect.top + windowRect.Height());
				CRect intersectRect;
				intersectRect.IntersectRect(monitorRect, newRect);

				//MoveWindow(HWND(msg->hwnd), intersectRect.left, intersectRect.top, intersectRect.Width(), intersectRect.Height(), TRUE);
				::SetWindowPos(msg->hwnd, NULL, intersectRect.left, intersectRect.top, 0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE );
				::SetWindowPos(msg->hwnd, NULL, 0, 0, intersectRect.Width(), intersectRect.Height(), SWP_NOREPOSITION | SWP_NOZORDER | SWP_NOACTIVATE );
				tmp.Format(" monitorRect %d %d %d %d", monitorRect.left, monitorRect.top, monitorRect.right, monitorRect.bottom);
				//AfxMessageBox(tmp,IDOK);
				tmp.Format("newRect %d %d %d %d", newRect.left, newRect.top, newRect.right, newRect.bottom);
				//AfxMessageBox(tmp,IDOK);
				tmp.Format("intersectRect %d %d %d %d", intersectRect.left, intersectRect.top, intersectRect.right, intersectRect.bottom);
				//AfxMessageBox(tmp,IDOK);

				//::GetWindowPlacement(HWND(msg->hwnd), &wPlacement);
				tmp.Format("%d %d %d %d", wPlacement.ptMinPosition.x, wPlacement.ptMinPosition.y, wPlacement.ptMaxPosition.x, wPlacement.ptMaxPosition.y);
				//AfxMessageBox(tmp,IDOK);

				wPlacement.showCmd = oldShowCmd;
				wPlacement.ptMinPosition = monitorRect.TopLeft();
				::SetWindowPlacement(HWND(msg->hwnd), &wPlacement);
				tmp.Format("%d %d %d %d", wPlacement.ptMinPosition.x, wPlacement.ptMinPosition.y, wPlacement.ptMaxPosition.x, wPlacement.ptMaxPosition.y);
				//AfxMessageBox(tmp,IDOK);
*/			}
		}
	}

	return CallNextHookEx(g_hookMenu, nCode, wParam, lParam);
}

// CMonitor

// constucts a monitor class not attached to any handle
CMonitor::CMonitor() : m_hMonitor(NULL)
{
}


// copy constructor
CMonitor::CMonitor(const CMonitor& monitor)
{
	m_hMonitor = (HMONITOR)monitor;
}

CMonitor::~CMonitor()
{
}



void CMonitor::Attach(const HMONITOR hMonitor)
{
	m_hMonitor = hMonitor;
}

HMONITOR CMonitor::Detach()
{
	HMONITOR hMonitor = m_hMonitor;
	m_hMonitor = NULL;
	return hMonitor;
}

void CMonitor::GetMonitorRect(LPRECT lprc) const
{
	MONITORINFO mi;
	RECT rc;

	mi.cbSize = sizeof(mi);
	::GetMonitorInfo(m_hMonitor, &mi);
	rc = mi.rcMonitor;

	::SetRect(lprc, rc.left, rc.top, rc.right, rc.bottom);
}

//
// these methods return true if any part of the item intersects the monitor rect
BOOL CMonitor::IsOnMonitor( const POINT pt ) const
{
	CRect rect;
	GetMonitorRect( rect );

	return rect.PtInRect( pt );
}

BOOL CMonitor::IsOnMonitor( const CWnd* pWnd ) const
{
	CRect rect;
	GetMonitorRect( rect );

	ASSERT( ::IsWindow( pWnd->GetSafeHwnd() ) );
	CRect wndRect;
	pWnd->GetClientRect( &wndRect );
	pWnd->ClientToScreen(&wndRect);

	return rect.IntersectRect( rect, wndRect );
}

BOOL CMonitor::IsOnMonitor( const LPRECT lprc ) const
{
	CRect rect;
	GetMonitorRect( rect );

	return rect.IntersectRect( rect, lprc );
}

void CMonitor::CenterRectToMonitor( LPRECT lprc, const BOOL UseWorkAreaRect ) const
{
    int  w = lprc->right - lprc->left;
    int  h = lprc->bottom - lprc->top;

    CRect rect;
    if ( UseWorkAreaRect )
        GetWorkAreaRect( &rect );
    else
        GetMonitorRect( &rect );

    lprc->left = rect.left + ( rect.Width() - w ) / 2;
    lprc->top = rect.top + ( rect.Height() - h ) / 2;
    lprc->right	= lprc->left + w;
    lprc->bottom = lprc->top + h;
}

void CMonitor::CenterWindowToMonitor( CWnd* const pWnd, const BOOL UseWorkAreaRect ) const
{
	ASSERT( pWnd );
	ASSERT( ::IsWindow( pWnd->m_hWnd ) );

	CRect rect;
    pWnd->GetWindowRect( &rect );
    CenterRectToMonitor( &rect, UseWorkAreaRect );
    CRect monRect;
    if ( UseWorkAreaRect )
        GetWorkAreaRect( &monRect );
    else
        GetMonitorRect( &monRect );
	CRect sRect(rect);
	::IntersectRect(&rect, &sRect, &monRect);

	pWnd->SetWindowPos( NULL, 0, 0, rect.Width(), rect.Height(), SWP_NOREPOSITION | SWP_NOZORDER | SWP_NOACTIVATE );
    pWnd->SetWindowPos( NULL, rect.left, rect.top, 0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE );
}

//
// the work area does not include the start bar
void CMonitor::GetWorkAreaRect( LPRECT lprc ) const
{
	MONITORINFO mi;
    RECT        rc;

	mi.cbSize = sizeof( mi );
	::GetMonitorInfo( m_hMonitor, &mi );
	rc = mi.rcWork;

	::SetRect( lprc, rc.left, rc.top, rc.right, rc.bottom );
}

