#include "StdAfx.h"
#include "Registry.h"
#include "resource.h"
#include "calendar.h"
#include "builddesktopimage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CBuildDesktopImage::CBuildDesktopImage(HWND hWnd)
{
	this->hWnd = hWnd;
	hBitmap = NULL;
	m_nYPos = m_nXPos = 100;
	m_nTransparency = 125;
	ReadRegistry();
}

CBuildDesktopImage::~CBuildDesktopImage(void)
{
}

void CBuildDesktopImage::SetHWnd(HWND hWnd)
{
	this->hWnd = hWnd;
}

void CBuildDesktopImage::SetIcon()
{
	CTime tm = CTime::GetCurrentTime();
	CString date = tm.Format("%A %d %B %Y");
	date.Replace("Monday", "Lundi");
	date.Replace("Tuesday", "Mardi");
	date.Replace("Wednesday", "Mercredi");
	date.Replace("Thursday", "Jeudi");
	date.Replace("Friday", "Vendredi");
	date.Replace("Saturday", "Samedi");
	date.Replace("Sunday", "Dimanche");
	date.Replace("January", "Janvier");
	date.Replace("February", "Fvrier");
	date.Replace("March", "Mars");
	date.Replace("April", "Avril");
	date.Replace("May", "Mai");
	date.Replace("June", "Juin");
	date.Replace("July", "Juillet");
	date.Replace("August", "Aot");
	date.Replace("September", "September");
	date.Replace("October", "Octobre");
	date.Replace("November", "Novembre");
	date.Replace("December", "Dcembre");
	HICON hIcon = AfxGetApp()->LoadIcon(IDI_ICON1 + (tm.GetDay() - 1));

	if (hIcon)
	{
		if (NULL != m_hCalIcon)
			::DestroyIcon(m_hCalIcon);
		m_hCalIcon = hIcon;
		m_tnd.cbSize = sizeof(NOTIFYICONDATA);
		m_tnd.hWnd = this->hWnd;
		m_tnd.uID = IDR_MAINFRAME;
		m_tnd.hIcon = m_hCalIcon;
		m_tnd.uFlags = NIF_TIP;
		strcpy(m_tnd.szTip, date.operator LPCTSTR());
		m_tnd.uCallbackMessage = WM_NI_MSG;
		m_tnd.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
		Shell_NotifyIcon(NIM_MODIFY, &m_tnd);
	}
}

HBITMAP CBuildDesktopImage::LoadImage(CString szFile, long& lWidth, long& lHeight)
{
	HANDLE hFile;
	HBITMAP hBmp;
	DWORD dwSize;
	DWORD dwRead;
	HGLOBAL hMemJpeg;
	LPSTREAM lpStream;
	OLE_HANDLE hJpegBmp;
	HRESULT hr;
	LPPICTURE lpPicture = NULL;
	void* pMemJpeg;

	/* Open the file and get the size. */
	if ((hFile = CreateFile(szFile, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL)) == INVALID_HANDLE_VALUE)
		return NULL;

	if ((dwSize = GetFileSize(hFile, NULL)) == 0xFFFFFFFF)
	{
		CloseHandle(hFile);
		return (NULL);
	}

	/* Allocate space for file, read it in, and then close the file again. */
	if ((hMemJpeg = GlobalAlloc(GMEM_MOVEABLE, dwSize)) == NULL)
	{
		CloseHandle(hFile);
		return (NULL);
	}

	if ((pMemJpeg = GlobalLock(hMemJpeg)) == NULL)
	{
		CloseHandle(hFile);
		GlobalFree(hMemJpeg);
		return (NULL);
	}

	if (!ReadFile(hFile, pMemJpeg, dwSize, &dwRead, NULL))
	{
		CloseHandle(hFile);
		GlobalFree(hMemJpeg);
		return (NULL);
	}

	CloseHandle(hFile);
	GlobalUnlock(hMemJpeg);


	/* Create the stream and load the picture. */
	if ((hr = CreateStreamOnHGlobal(hMemJpeg, TRUE, &lpStream)) != S_OK)
	{
		GlobalFree(hMemJpeg);
		return (NULL);
	}

	if (::OleLoadPicture(lpStream, dwSize, FALSE, IID_IPicture, (LPVOID *)&lpPicture) != S_OK)
	{
		GlobalFree(hMemJpeg);
		lpStream->Release();
		return (NULL);
	}

	/* Get the handle to the image, and then copy it. */
	if ((lpPicture->get_Handle(&hJpegBmp)) != S_OK)
	{
		GlobalFree(hMemJpeg);
		lpStream->Release();
		lpPicture->Release();
		return (NULL);
	}
	lWidth = lHeight = 0;
	lpPicture->get_Width((OLE_XSIZE_HIMETRIC *)&lWidth);
	lpPicture->get_Height((OLE_XSIZE_HIMETRIC *)&lHeight);

	if ((hBmp = (HBITMAP)CopyImage((HANDLE *)hJpegBmp, IMAGE_BITMAP, 0, 0, LR_COPYRETURNORG)) == NULL)
	{
		GlobalFree(hMemJpeg);
		lpStream->Release();
		lpPicture->Release();
		return (NULL);
	}

	/* Free the original image and memory. */
	GlobalFree(hMemJpeg);
	lpStream->Release();
	lpPicture->Release();

	return (hBmp);
}

BOOL CBuildDesktopImage::SaveBitmap(HDC hDC, HBITMAP hBitmap, CString szPath)
{
	FILE* fp = NULL;
	fp = fopen(szPath.GetBuffer(1), "wb");
	if (fp == NULL)
		return false;

	BITMAP Bm;
	BITMAPINFO BitInfo;
	ZeroMemory(&BitInfo, sizeof(BITMAPINFO));
	BitInfo.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	BitInfo.bmiHeader.biBitCount = 0;

	if (!::GetDIBits(hDC, hBitmap, 0, 0, NULL, &BitInfo, DIB_RGB_COLORS))
		return (false);

	Bm.bmHeight = BitInfo.bmiHeader.biHeight;
	Bm.bmWidth = BitInfo.bmiHeader.biWidth;

	BITMAPFILEHEADER BmHdr;

	BmHdr.bfType = 0x4d42;   // 'BM' WINDOWS_BITMAP_SIGNATURE
	BmHdr.bfSize = (((3 * Bm.bmWidth + 3) & ~3) * Bm.bmHeight) + sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);
	BmHdr.bfReserved1 = BmHdr.bfReserved2 = 0;
	BmHdr.bfOffBits = (DWORD) sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);

	BitInfo.bmiHeader.biCompression = 0;
	// Writing Bitmap File Header ////
	fwrite(&BmHdr, sizeof(BITMAPFILEHEADER), 1, fp);

	fwrite(&BitInfo.bmiHeader, sizeof(BITMAPINFOHEADER), 1, fp);

	BYTE* pData = new BYTE[BitInfo.bmiHeader.biSizeImage + 5];
	if (!::GetDIBits(hDC, hBitmap, 0, Bm.bmHeight, pData, &BitInfo, DIB_RGB_COLORS))
		return (false);
	if (pData != NULL)
		fwrite(pData, 1, BitInfo.bmiHeader.biSizeImage, fp);

	fclose(fp);
	delete (pData);

	return (true);
}

void CBuildDesktopImage::ReadRegistry()
{
	m_szFontName = CRegistry::GetRegString("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Font Name", _T("Monotype Corsiva"));
	m_nFontSize = CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Text Size", 24);
	m_crTextColor = CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Text Color", (INT)RGB(0, 255, 0));
	m_crBackColor = CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Back Ground Color", (INT)RGB(255, 255, 255));
	m_nTransparency = CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Transparency", 88);
	m_nBackgroundTransparency = CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "BackgroundTransparency", 0);
	m_nXPos = CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "StartX", 100);
	m_nYPos = CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "StartY", 100);
}

void CBuildDesktopImage::WriteRegistry()
{
	CString szSection(_T("Configuration"));
	CRegistry::SetRegString("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Font Name", m_szFontName);
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Text Size", m_nFontSize);
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Text Color", m_crTextColor);
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Back Ground Color", m_crBackColor);
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "Transparency", m_nTransparency);
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "BackgroundTransparency", m_nBackgroundTransparency);
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "StartX", m_nXPos);
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "StartY", m_nYPos);
}

HBITMAP Create24BPPDIBSection(HDC hDC, int iWidth, int iHeight)
{
	BITMAPINFO bmi;
	HBITMAP hbm;
	LPBYTE pBits;

	// Initialize header to 0s.
	ZeroMemory(&bmi, sizeof(bmi));

	// Fill out the fields you care about.
	bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bmi.bmiHeader.biWidth = iWidth;
	bmi.bmiHeader.biHeight = iHeight;
	bmi.bmiHeader.biPlanes = 1;
	bmi.bmiHeader.biBitCount = 24;
	bmi.bmiHeader.biCompression = BI_RGB;

	// Create the surface.
	hbm = CreateDIBSection(hDC, &bmi, DIB_RGB_COLORS, (void**)&pBits, NULL, 0);

	return(hbm);
}

BOOL BitmapsCompatible(LPBITMAP lpbm1, LPBITMAP lpbm2)
{
	if (lpbm1->bmBitsPixel != lpbm2->bmBitsPixel)
		return FALSE;
	if (lpbm1->bmPlanes != lpbm2->bmPlanes)
		return FALSE;
	if (lpbm1->bmWidth != lpbm2->bmWidth)
		return FALSE;
	if (lpbm1->bmHeight != lpbm2->bmHeight)
		return FALSE;
	return TRUE;
}

BOOL BlendImages(HBITMAP hbmSrc1, HBITMAP hbmSrc2, HBITMAP hbmDst, DWORD dwWeight1, COLORREF m_crBackColor, BOOL backgroundTransparency)
{
	BITMAP bmSrc1, bmSrc2, bmDst;
	RGBTRIPLE* lprgbSrc1, * lprgbSrc2, * lprgbDst;
	DWORD dwWidthBytes, dwWeight2;
	int x, y;

	// Only values between 0 and 255 are valid.
	if (dwWeight1 > 255)
		return FALSE;

	// Get weighting value for second source image.
	dwWeight2 = 255 - dwWeight1;

	// Get information about the surfaces you were passed.
	if (!GetObject(hbmSrc1, sizeof(BITMAP), &bmSrc1))
		return FALSE;
	if (!GetObject(hbmSrc2, sizeof(BITMAP), &bmSrc2))
		return FALSE;
	if (!GetObject(hbmDst, sizeof(BITMAP), &bmDst))
		return FALSE;

	// Make sure you have data that meets your requirements.
	if (!BitmapsCompatible(&bmSrc1, &bmSrc2))
		return FALSE;
	if (!BitmapsCompatible(&bmSrc1, &bmDst))
		return FALSE;
	if (bmSrc1.bmBitsPixel != 24)
		return FALSE;
	if (bmSrc1.bmPlanes != 1)
		return FALSE;
	if (!bmSrc1.bmBits || !bmSrc2.bmBits || !bmDst.bmBits)
		return FALSE;

	dwWidthBytes = bmDst.bmWidthBytes;

	// Initialize the surface pointers.
	lprgbSrc1 = (RGBTRIPLE *)bmSrc1.bmBits; 
	lprgbSrc2 = (RGBTRIPLE *)bmSrc2.bmBits;  
	lprgbDst = (RGBTRIPLE *)bmDst.bmBits;

	for (y = 0; y < bmDst.bmHeight; y++)
	{
		for (x = 0; x < bmDst.bmWidth; x++)
		{
			if (backgroundTransparency && m_crBackColor == RGB(lprgbSrc1[x].rgbtRed, lprgbSrc1[x].rgbtGreen, lprgbSrc1[x].rgbtBlue))
			{
				lprgbDst[x].rgbtRed = lprgbSrc2[x].rgbtRed;
				lprgbDst[x].rgbtGreen = lprgbSrc2[x].rgbtGreen;
				lprgbDst[x].rgbtBlue = lprgbSrc2[x].rgbtBlue;
			}
			else
			{
				lprgbDst[x].rgbtRed = (BYTE)((((DWORD)lprgbSrc1[x].rgbtRed * dwWeight1) + ((DWORD)lprgbSrc2[x].rgbtRed * dwWeight2)) >> 8);
				lprgbDst[x].rgbtGreen = (BYTE)((((DWORD)lprgbSrc1[x].rgbtGreen * dwWeight1) + ((DWORD)lprgbSrc2[x].rgbtGreen * dwWeight2)) >> 8);
				lprgbDst[x].rgbtBlue = (BYTE)((((DWORD)lprgbSrc1[x].rgbtBlue * dwWeight1) + ((DWORD)lprgbSrc2[x].rgbtBlue * dwWeight2)) >> 8);
			}
		}

		// Move to next scan line.
		lprgbSrc1 = (RGBTRIPLE *)((LPBYTE)lprgbSrc1 + dwWidthBytes);
		lprgbSrc2 = (RGBTRIPLE *)((LPBYTE)lprgbSrc2 + dwWidthBytes);
		lprgbDst = (RGBTRIPLE *)((LPBYTE)lprgbDst + dwWidthBytes);
	}

	return TRUE;
}

BOOL DoAlphaBlend(HDC hdcDest, // Handle to destination DC.
  int nXOriginDest, // X-coord of upper-left corner.
  int nYOriginDest, // Y-coord of upper-left corner.
  int nWidthDest, // Destination width.
  int nHeightDest, // Destination height.
  HDC hdcSrc, // Handle to source DC.
  int nXOriginSrc, // X-coord of upper-left corner.
  int nYOriginSrc, // Y-coord of upper-left corner.
  int nWidthSrc, // Source width.
  int nHeightSrc, // Source height.
  DWORD dwSourceWeight, // Source weighting (between 0 and 255).
  COLORREF m_crBackColor, // Background color
  BOOL backgroundTransparency) // BackgroundTransparency
{
	HDC hdcSrc1 = NULL;
	HDC hdcSrc2 = NULL;
	HDC hdcDst = NULL;
	HBITMAP hbmSrc1 = NULL;
	HBITMAP hbmSrc2 = NULL;
	HBITMAP hbmDst = NULL;
	BOOL bReturn;

	// Create surfaces for sources and destination images.
	hbmSrc1 = Create24BPPDIBSection(hdcDest, nWidthDest, nHeightDest);
	if (!hbmSrc1)
		goto HANDLEERROR;

	hbmSrc2 = Create24BPPDIBSection(hdcDest, nWidthDest, nHeightDest);
	if (!hbmSrc2)
		goto HANDLEERROR;

	hbmDst = Create24BPPDIBSection(hdcDest, nWidthDest, nHeightDest);
	if (!hbmDst)
		goto HANDLEERROR;


	// Create HDCs to hold our surfaces.
	hdcSrc1 = CreateCompatibleDC(hdcDest);
	if (!hdcSrc1)
		goto HANDLEERROR;

	hdcSrc2 = CreateCompatibleDC(hdcDest);
	if (!hdcSrc2)
		goto HANDLEERROR;

	hdcDst = CreateCompatibleDC(hdcDest);
	if (!hdcDst)
		goto HANDLEERROR;


	// Prepare the surfaces for drawing.
	SelectObject(hdcSrc1, hbmSrc1);
	SelectObject(hdcSrc2, hbmSrc2);
	SelectObject(hdcDst, hbmDst);
	SetStretchBltMode(hdcSrc1, COLORONCOLOR);
	SetStretchBltMode(hdcSrc2, COLORONCOLOR);


	// Capture a copy of the source area.
	if (!StretchBlt(hdcSrc1, 0, 0, nWidthDest, nHeightDest, hdcSrc, nXOriginSrc, nYOriginSrc, nWidthSrc, nHeightSrc, SRCCOPY))
		goto HANDLEERROR;

	// Capture a copy of the destination area.
	if (!StretchBlt(hdcSrc2, 0, 0, nWidthDest, nHeightDest, hdcDest, nXOriginDest, nYOriginDest, nWidthDest, nHeightDest, SRCCOPY))
		goto HANDLEERROR;


	// Blend the two source areas to create the destination image.
	bReturn = BlendImages(hbmSrc1, hbmSrc2, hbmDst, dwSourceWeight, m_crBackColor, backgroundTransparency);


	// Clean up objects you do not need any longer.
	// You cannot delete an object that's selected into an
	// HDC so delete the HDC first. 
	DeleteDC(hdcSrc1);
	DeleteDC(hdcSrc2);
	DeleteObject(hbmSrc1);
	DeleteObject(hbmSrc2);


	// Display the blended (destination) image to the target HDC.
	if (bReturn)
	{
		BitBlt(hdcDest, nXOriginDest, nYOriginDest, nWidthDest, nHeightDest, hdcDst, 0, 0, SRCCOPY);
	}


	// Clean up the rest of the objects you created.
	DeleteDC(hdcDst);
	DeleteObject(hbmDst);

	return bReturn;

	HANDLEERROR:

	if (hdcSrc1)
		DeleteDC(hdcSrc1);
	if (hdcSrc2)
		DeleteDC(hdcSrc2);
	if (hdcDst)
		DeleteDC(hdcDst);
	if (hbmSrc1)
		DeleteObject(hbmSrc1);
	if (hbmSrc2)
		DeleteObject(hbmSrc2);
	if (hbmDst)
		DeleteObject(hbmDst);

	return FALSE;
}

void CBuildDesktopImage::StartBitmap()
{
	// Delete the last bitmap
	if (hBitmap != NULL)
	{
		DeleteObject(hBitmap);
		hBitmap = NULL;
	}

	// Screen Resolution //
	nVirtualScreenWidth = GetSystemMetrics(SM_CXVIRTUALSCREEN);
	nVirtualScreenHeight = GetSystemMetrics(SM_CYVIRTUALSCREEN);
	hDC = ::GetDC(hWnd);

	m_MonitorArray.SetSize(::GetSystemMetrics(SM_CMONITORS));
	ADDMONITOR addMonitor;
	addMonitor.pMonitors = &m_MonitorArray;
	addMonitor.currentIndex = 0;
	::EnumDisplayMonitors(NULL, NULL, AddMonitorsCallBack, (LPARAM)&addMonitor);


	hMemDC = ::CreateCompatibleDC(hDC);
	hBitmap = CreateCompatibleBitmap(hDC, nVirtualScreenWidth, nVirtualScreenHeight);
	::SelectObject(hMemDC, hBitmap);	

	COLORREF crDeskColor = GetSysColor(COLOR_DESKTOP);
	CBrush FillBrush;
	FillBrush.CreateSolidBrush(crDeskColor);
	CRect Rect(0, 0, nVirtualScreenWidth, nVirtualScreenHeight);
	::FillRect(hMemDC, &Rect, FillBrush.operator HBRUSH());
}

#define HIMETRIC_INCH 2540

void CBuildDesktopImage::SetImage(CString img, int style, int monitor)
{
	LONG lWidth = 0,lHeight = 0;

	if (hBitmap)
	{
		// Image Height and Width

		SetStretchBltMode(hMemDC, HALFTONE);

		HDC hBitmapDC = ::CreateCompatibleDC(hDC);
		CMonitor* mon = (CMonitor*)m_MonitorArray.GetAt(monitor);
		CRect ecran;
		mon->GetMonitorRect(&ecran);

		HBITMAP hMainBitmap = LoadImage(img, lWidth, lHeight);
		if (hMainBitmap != NULL)
		{
			::SelectObject(hBitmapDC, hMainBitmap);	
			lWidth = MulDiv(lWidth, GetDeviceCaps(hDC, LOGPIXELSX), HIMETRIC_INCH);
			lHeight = MulDiv(lHeight, GetDeviceCaps(hDC, LOGPIXELSY), HIMETRIC_INCH);

			::SelectObject(hBitmapDC, hMainBitmap);	

			CRect rectangle1, rectangle2;
			HDC hMemTempDC = ::CreateCompatibleDC(hDC);

			HBITMAP hTempBitmap = CreateCompatibleBitmap(hDC, ecran.Width(), ecran.Height());
			::SelectObject(hMemTempDC, hTempBitmap);	
			COLORREF crDeskColor = GetSysColor(COLOR_DESKTOP);
			CBrush FillBrush;
			FillBrush.CreateSolidBrush(crDeskColor);
			CRect Rect(0, 0, ecran.Width(), ecran.Height());
			::FillRect(hMemTempDC, &Rect, FillBrush.operator HBRUSH());
			SetStretchBltMode(hMemTempDC, HALFTONE);
			if (style == 0)
			{
				// Tiled
				rectangle1.top = 0;
				rectangle1.left = 0;
				rectangle1.bottom = lHeight;
				rectangle1.right = lWidth;
				for (int xPos = 0; xPos < ecran.Width(); xPos += lWidth)
				{
					for (int yPos = 0; yPos < ecran.Height(); yPos += lHeight)
					{
						::BitBlt(hMemTempDC, xPos, yPos, rectangle1.Width(), rectangle1.Height(), hBitmapDC, 0, 0, SRCCOPY);
					}
				}
			}
			if (style == 1)
			{
				// Center
				rectangle1.top = 0;
				rectangle1.left = 0;
				rectangle1.bottom = lHeight;
				rectangle1.right = lWidth;
				if (ecran.Width() > rectangle1.Width())
				{
					if (ecran.Height() > rectangle1.Height())
					{
						::BitBlt(hMemTempDC, (ecran.Width() - rectangle1.Width()) / 2, (ecran.Height() - rectangle1.Height()) / 2, rectangle1.Width(), rectangle1.Height(), hBitmapDC, 0, 0, SRCCOPY);
					}
					else
					{
						::BitBlt(hMemTempDC, (ecran.Width() - rectangle1.Width()) / 2, 0, rectangle1.Width(), rectangle1.Height(), hBitmapDC, 0, 0, SRCCOPY);
					}
				}
				else
				{
					if (ecran.Height() > rectangle1.Height())
					{
						::BitBlt(hMemTempDC, 0, 0, ecran.Width(), ecran.Height(), hBitmapDC, 0, 0, SRCCOPY);
					}
					else
					{
						::BitBlt(hMemTempDC, 0, 0, rectangle1.Width(), rectangle1.Height(), hBitmapDC, 0, 0, SRCCOPY);
					}
				}
			}
			if (style == 2)
			{
				// Stretch
				::StretchBlt(hMemTempDC, 0, 0, ecran.Width(), ecran.Height(), hBitmapDC, 0, 0, lWidth, lHeight, SRCCOPY);
			} //Stretch

			if (ecran.left >= 0)
			{
				if (ecran.top >= 0)
				{
					::BitBlt(hMemDC, ecran.left, ecran.top, ecran.Width(), ecran.Height(), hMemTempDC, 0, 0, SRCCOPY);
				}
				else
				{
					::BitBlt(hMemDC, ecran.left, ecran.top, ecran.Width(), ecran.Height(), hMemTempDC, 0, 0, SRCCOPY);
					::BitBlt(hMemDC, ecran.left, nVirtualScreenHeight + ecran.top, ecran.Width(), ecran.Height(), hMemTempDC, 0, 0, SRCCOPY);
				}
			}
			else
			{
				if (ecran.top >= 0)
				{
					::BitBlt(hMemDC, ecran.left, ecran.top, ecran.Width(), ecran.Height(), hMemTempDC, 0, 0, SRCCOPY);
					::BitBlt(hMemDC, nVirtualScreenWidth + ecran.left, ecran.top, ecran.Width(), ecran.Height(), hMemTempDC, 0, 0, SRCCOPY);
				}
				else
				{
					::BitBlt(hMemDC, ecran.left, ecran.top, ecran.Width(), ecran.Height(), hMemTempDC, 0, 0, SRCCOPY);
					::BitBlt(hMemDC, ecran.left, nVirtualScreenHeight + ecran.top, ecran.Width(), ecran.Height(), hMemTempDC, 0, 0, SRCCOPY);
					::BitBlt(hMemDC, nVirtualScreenWidth + ecran.left, ecran.top, ecran.Width(), ecran.Height(), hMemTempDC, 0, 0, SRCCOPY);
					::BitBlt(hMemDC, nVirtualScreenWidth + ecran.left, nVirtualScreenHeight + ecran.top, ecran.Width(), ecran.Height(), hMemTempDC, 0, 0, SRCCOPY);
				}
			}
			DeleteDC(hMemTempDC);
			DeleteObject(hTempBitmap);
		}
		DeleteDC(hBitmapDC);
		DeleteObject(hMainBitmap);
		hBitmapDC = NULL;
		hMainBitmap = NULL;
	}
}
void CBuildDesktopImage::StopBitmap()
{
	if (hBitmap)
	{
		// Making Calendar
		C_Calendar* pCalendar;
		pCalendar = new C_Calendar();
		pCalendar->SetFont(m_szFontName);
		pCalendar->SetTextColor(m_crTextColor);
		pCalendar->SetBackColor(m_crBackColor);
		pCalendar->SetTextSize(m_nFontSize);
		HDC hCalDC = pCalendar->DrawCalendar(hDC);
		DoAlphaBlend(hMemDC, m_nXPos, m_nYPos, pCalendar->GetWidth(), pCalendar->GetHeight(), hCalDC, 0, 0, pCalendar->GetWidth(), pCalendar->GetHeight(), m_nTransparency, m_crBackColor, m_nBackgroundTransparency);

		CHAR szSysDir[MAX_PATH];
		GetSystemDirectory(szSysDir, MAX_PATH);
		CString szNewFile = szSysDir;
		szNewFile += _T("\\Desktop Calendar.bmp");

		SaveBitmap(hMemDC, hBitmap, szNewFile);

		delete(pCalendar);
		pCalendar = NULL;
		DeleteDC(hMemDC);
		DeleteObject(hBitmap);
		hBitmap = NULL;
		hMemDC = NULL;
		::ReleaseDC(hWnd, hDC);
	}

	SetIcon();

	while (m_MonitorArray.GetSize() != 0)
	{
		CMonitor* mon = (CMonitor*)m_MonitorArray.GetAt(0);
		delete mon;
		m_MonitorArray.RemoveAt(0);
	}
}
BOOL CALLBACK CBuildDesktopImage::AddMonitorsCallBack(HMONITOR hMonitor, HDC hdcMonitor, LPRECT lprcMonitor, LPARAM dwData)
{
	LPADDMONITOR pAddMonitor = (LPADDMONITOR)dwData;

	CMonitor* pMonitor = new CMonitor;
	pMonitor->Attach(hMonitor);

	pAddMonitor->pMonitors->SetAt(pAddMonitor->currentIndex, pMonitor);
	pAddMonitor->currentIndex++;

	return TRUE;
}

// CMonitor

// constucts a monitor class not attached to any handle
CMonitor::CMonitor() : m_hMonitor(NULL)
{
}


// copy constructor
CMonitor::CMonitor(const CMonitor& monitor)
{
	m_hMonitor = (HMONITOR)monitor;
}

CMonitor::~CMonitor()
{
}



void CMonitor::Attach(const HMONITOR hMonitor)
{
	m_hMonitor = hMonitor;
}

HMONITOR CMonitor::Detach()
{
	HMONITOR hMonitor = m_hMonitor;
	m_hMonitor = NULL;
	return hMonitor;
}

void CMonitor::GetMonitorRect(LPRECT lprc) const
{
	MONITORINFO mi;
	RECT rc;

	mi.cbSize = sizeof(mi);
	::GetMonitorInfo(m_hMonitor, &mi);
	rc = mi.rcMonitor;

	::SetRect(lprc, rc.left, rc.top, rc.right, rc.bottom);
}

