#include "stdafx.h"
#include "clock.h"
#include "resource.h"
#include "registry.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define WS_EX_LAYERED   		0x00080000
#define LWA_COLORKEY			0x00000001
#define LWA_ALPHA   			0x00000002


extern "C"
{
	WINUSERAPI BOOL
	WINAPI
	SetLayeredWindowAttributes(HWND hwnd, COLORREF crKey, BYTE bAlpha, DWORD dwFlags);
}

/////////////////////////////////////////////////////////////////////////////
// CClock
IMPLEMENT_DYNAMIC(CClock, CWnd)

BEGIN_MESSAGE_MAP(CClock, CWnd)
//{{AFX_MSG_MAP(CClock)
ON_WM_PAINT()
ON_WM_CREATE()
ON_WM_TIMER()
ON_WM_NCHITTEST()
ON_WM_SETTINGCHANGE()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

HANDLE m_hTimeThread = NULL;

LRESULT TimeTickThread(LPVOID param)
{
	HWND hMainWnd = (HWND)param;
	if (::IsWindow(hMainWnd))
	{
		while (1)
		{
			Sleep(1000);
			SetTimer(hMainWnd, WM_CLOCKTIMETICK, 0, 0);
		}
	}

	return 0;
}

// CClock construction/destruction

CClock::CClock()
{
	m_nHour = -1;
	m_nMin = -1;
	// initialize drawing objects
	m_objColor = CLOCK_COLOR1;
	m_dotColor = CLOCK_COLOR1;
	// dot pen
	m_bkDotPen = CreatePen(PS_SOLID, 1, m_dotColor);
	// pen
	m_bkPen = CreatePen(PS_SOLID, 1, m_objColor);
}

CClock::~CClock()
{
	if (this->IsWindowVisible())
		this->DestroyWindow();
}

void CClock::Start()
{
	RECT rect;
	rect.top = 0;
	rect.bottom = 30;
	rect.left = 0;
	rect.right = 100;

	WNDCLASS wndcls;
	memset(&wndcls, 0, sizeof(WNDCLASS));

	wndcls.lpfnWndProc = ::DefWindowProc;
	wndcls.hInstance = AfxGetInstanceHandle();
	wndcls.hIcon = ::LoadIcon(AfxGetApp()->m_hInstance, MAKEINTRESOURCE(IDR_MAINFRAME));
	wndcls.hCursor = ::LoadCursor(AfxGetApp()->m_hInstance, IDC_ARROW);
	wndcls.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
	wndcls.lpszMenuName = NULL;
	wndcls.lpszClassName = _T("Clock");

	if (!AfxRegisterClass(&wndcls))
		return;

	this->CreateEx(WS_EX_TOPMOST | WS_EX_TOOLWINDOW, _T("Clock"), _T("ClockClass"), WS_POPUP, 0, 10, 0, 0, NULL, NULL, NULL);


	InitDrawObj();
	DefineWndRegion(FALSE);

	static DWORD wID;
	DWORD dwStyle;
	dwStyle = GetWindowLong(this->m_hWnd, GWL_STYLE);
	// remove caption & menu bar, etc. //
	dwStyle &= ~(WS_DLGFRAME | WS_SYSMENU | WS_MINIMIZEBOX | WS_MAXIMIZEBOX | WS_BORDER | WS_SIZEBOX);
	wID = SetWindowLong(this->m_hWnd, GWL_ID, 0);
	SetWindowLong(this->m_hWnd, GWL_STYLE, dwStyle);
	SetWindowLong(this->m_hWnd, GWL_EXSTYLE, GetWindowLong(this->m_hWnd, GWL_EXSTYLE) ^ (WS_EX_LAYERED));

	int xPos = this->iXPosClock;
	int yPos = this->iYPosClock; // GetSystemMetrics(SM_CYSCREEN) - 44;

	::SetWindowPos(this->m_hWnd, HWND_TOPMOST, xPos, yPos, 120, 50, SWP_FRAMECHANGED);

	m_hTimeThread = ::CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)TimeTickThread, (LPVOID)this->m_hWnd, 0, &dwThreadId);

	::SetLayeredWindowAttributes(this->m_hWnd, RGB(0, 0, 0), 150, LWA_ALPHA);
	ShowWindowClock(this->iDisplayClock);

	this->UpdateWindow();

	if (m_hTimeThread)
		::ResumeThread(m_hTimeThread);
	else
		::PostMessage(this->m_hWnd, WM_DESTROY, 0, 0);
}

void CClock::ShowWindowClock(int displayclock)
{
	this->iDisplayClock = displayclock;
	if (this->iDisplayClock == TRUE)
		this->ShowWindow(SW_SHOW);
	else
		this->ShowWindow(SW_HIDE);
}

/////////////////////////////////////////////////////////////////////////////
// CClock diagnostics

int CClock::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	ReadRegistry();
	return 0;
}

void CClock::OnTimer(UINT nIDEvent)
{
	switch (nIDEvent)
	{
	case WM_CLOCKTIMETICK:
		KillTimer(WM_CLOCKTIMETICK);
		OnTimeTick();
		break;
	}
}

void CClock::ReadRegistry()
{
	iDisplayClock = CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "DisplayClock", FALSE);
	iXPosClock = CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "StartXClock", GetSystemMetrics(SM_CXSCREEN) - 150);
	iYPosClock = CRegistry::GetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "StartYClock", 25);
}

void CClock::WriteRegistry()
{
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "DisplayClock", (DWORD)iDisplayClock);
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "StartXClock", (DWORD)iXPosClock);
	CRegistry::SetRegDWORD("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration", "StartYClock", (DWORD)iYPosClock);
}

BOOL CClock::DestroyWindow()
{
	RECT rect;
	GetWindowRect(&rect);
	iXPosClock = rect.left;
	iYPosClock = rect.top;

	WriteRegistry();
	return CWnd::DestroyWindow();
}

void CClock::InitDrawObj()
{
	// 1
	m_obj[0][0].cType = 1;
	m_obj[0][0].xPos = 3;
	m_obj[0][0].yPos = 0;
	// 5
	m_obj[0][5].cType = 2;
	m_obj[0][5].xPos = 0;
	m_obj[0][5].yPos = 3;
	// 6
	m_obj[0][1].cType = 2;
	m_obj[0][1].xPos = 19;
	m_obj[0][1].yPos = 3;
	// 2
	m_obj[1][0].cType = 1;
	m_obj[1][0].xPos = 31;
	m_obj[1][0].yPos = 0;
	// 3
	m_obj[2][0].cType = 1;
	m_obj[2][0].xPos = 67;
	m_obj[2][0].yPos = 0;
	// 4
	m_obj[3][0].cType = 1;
	m_obj[3][0].xPos = 95;
	m_obj[3][0].yPos = 0;
	// 7
	m_obj[1][5].cType = 2;
	m_obj[1][5].xPos = 28;
	m_obj[1][5].yPos = 3;
	// 8
	m_obj[1][1].cType = 2;
	m_obj[1][1].xPos = 47;
	m_obj[1][1].yPos = 3;
	// 9
	m_obj[2][5].cType = 2;
	m_obj[2][5].xPos = 64;
	m_obj[2][5].yPos = 3;
	// 10
	m_obj[2][1].cType = 2;
	m_obj[2][1].xPos = 83;
	m_obj[2][1].yPos = 3;
	// 11
	m_obj[3][5].cType = 2;
	m_obj[3][5].xPos = 92;
	m_obj[3][5].yPos = 3;
	// 12
	m_obj[3][1].cType = 2;
	m_obj[3][1].xPos = 111;
	m_obj[3][1].yPos = 3;
	// 13
	m_obj[0][6].cType = 1;
	m_obj[0][6].xPos = 3;
	m_obj[0][6].yPos = 19;
	// 14
	m_obj[1][6].cType = 1;
	m_obj[1][6].xPos = 31;
	m_obj[1][6].yPos = 19;
	// 15
	m_obj[2][6].cType = 1;
	m_obj[2][6].xPos = 67;
	m_obj[2][6].yPos = 19;
	// 16
	m_obj[3][6].cType = 1;
	m_obj[3][6].xPos = 95;
	m_obj[3][6].yPos = 19;
	// 17
	m_obj[0][4].cType = 2;
	m_obj[0][4].xPos = 0;
	m_obj[0][4].yPos = 22;
	// 18
	m_obj[0][2].cType = 2;
	m_obj[0][2].xPos = 19;
	m_obj[0][2].yPos = 22;
	// 19
	m_obj[1][4].cType = 2;
	m_obj[1][4].xPos = 28;
	m_obj[1][4].yPos = 22;
	// 20
	m_obj[1][2].cType = 2;
	m_obj[1][2].xPos = 47;
	m_obj[1][2].yPos = 22;
	// 21
	m_obj[2][4].cType = 2;
	m_obj[2][4].xPos = 64;
	m_obj[2][4].yPos = 22;
	// 22
	m_obj[2][2].cType = 2;
	m_obj[2][2].xPos = 83;
	m_obj[2][2].yPos = 22;
	// 23
	m_obj[3][4].cType = 2;
	m_obj[3][4].xPos = 92;
	m_obj[3][4].yPos = 22;
	// 24
	m_obj[3][2].cType = 2;
	m_obj[3][2].xPos = 111;
	m_obj[3][2].yPos = 22;
	// 25
	m_obj[0][3].cType = 1;
	m_obj[0][3].xPos = 3;
	m_obj[0][3].yPos = 38;
	// 26
	m_obj[1][3].cType = 1;
	m_obj[1][3].xPos = 31;
	m_obj[1][3].yPos = 38;
	// 27
	m_obj[2][3].cType = 1;
	m_obj[2][3].xPos = 67;
	m_obj[2][3].yPos = 38;
	// 28
	m_obj[3][3].cType = 1;
	m_obj[3][3].xPos = 95;
	m_obj[3][3].yPos = 38;

	/*	
			   0				   0
			 *****				 *****
			*	  *				*	  *
		  5	*	  * 1		  5	*	  * 1
			*  6  *				*  6  *
			 *****				 *****
	  		*	  *				*	  *
		  4 *	  * 2		  4 *	  * 2
			*	  *				*	  *
			 *****				 *****
			   3				   3 
			1st BYTE			2nd BYTE
	*/

	// HOUR
	m_objHour[0] = (63 << 8) + 63;		// 00
	m_objHour[1] = (63 << 8) + 6;		// 01
	m_objHour[2] = (63 << 8) + 91;		// 02
	m_objHour[3] = (63 << 8) + 79;		// 03
	m_objHour[4] = (63 << 8) + 102;		// 04
	m_objHour[5] = (63 << 8) + 109;		// 05
	m_objHour[6] = (63 << 8) + 125;		// 06
	m_objHour[7] = (63 << 8) + 7;		// 07
	m_objHour[8] = (63 << 8) + 127;		// 08
	m_objHour[9] = (63 << 8) + 111;		// 09
	m_objHour[10] = (6 << 8) + 63;		// 10
	m_objHour[11] = (6 << 8) + 6;		// 11
	m_objHour[12] = (6 << 8) + 91;		// 12
	m_objHour[13] = (6 << 8) + 79;		// 13
	m_objHour[14] = (6 << 8) + 102;		// 14
	m_objHour[15] = (6 << 8) + 109;		// 15
	m_objHour[16] = (6 << 8) + 125;		// 16
	m_objHour[17] = (6 << 8) + 7;		// 17
	m_objHour[18] = (6 << 8) + 127;		// 18
	m_objHour[19] = (6 << 8) + 111;		// 19
	m_objHour[20] = (91 << 8) + 63;		// 20
	m_objHour[21] = (91 << 8) + 6;		// 21
	m_objHour[22] = (91 << 8) + 91;		// 22
	m_objHour[23] = (91 << 8) + 79;		// 23

	// MINUTE
	m_objMin[0] = (63 << 8) + 63;
	m_objMin[1] = (63 << 8) + 6;
	m_objMin[2] = (63 << 8) + 91;
	m_objMin[3] = (63 << 8) + 79;
	m_objMin[4] = (63 << 8) + 102;
	m_objMin[5] = (63 << 8) + 109;
	m_objMin[6] = (63 << 8) + 125;
	m_objMin[7] = (63 << 8) + 7;
	m_objMin[8] = (63 << 8) + 127;
	m_objMin[9] = (63 << 8) + 111;
	m_objMin[10] = (6 << 8) + 63;
	m_objMin[11] = (6 << 8) + 6;
	m_objMin[12] = (6 << 8) + 91;
	m_objMin[13] = (6 << 8) + 79;
	m_objMin[14] = (6 << 8) + 102;
	m_objMin[15] = (6 << 8) + 109;
	m_objMin[16] = (6 << 8) + 125;
	m_objMin[17] = (6 << 8) + 7;
	m_objMin[18] = (6 << 8) + 127;
	m_objMin[19] = (6 << 8) + 111;
	m_objMin[20] = (91 << 8) + 63;
	m_objMin[21] = (91 << 8) + 6;
	m_objMin[22] = (91 << 8) + 91;
	m_objMin[23] = (91 << 8) + 79;
	m_objMin[24] = (91 << 8) + 102;
	m_objMin[25] = (91 << 8) + 109;
	m_objMin[26] = (91 << 8) + 125;
	m_objMin[27] = (91 << 8) + 7;
	m_objMin[28] = (91 << 8) + 127;
	m_objMin[29] = (91 << 8) + 111;
	m_objMin[30] = (79 << 8) + 63;
	m_objMin[31] = (79 << 8) + 6;
	m_objMin[32] = (79 << 8) + 91;
	m_objMin[33] = (79 << 8) + 79;
	m_objMin[34] = (79 << 8) + 102;
	m_objMin[35] = (79 << 8) + 109;
	m_objMin[36] = (79 << 8) + 125;
	m_objMin[37] = (79 << 8) + 7;
	m_objMin[38] = (79 << 8) + 127;
	m_objMin[39] = (79 << 8) + 111;
	m_objMin[40] = (102 << 8) + 63;
	m_objMin[41] = (102 << 8) + 6;
	m_objMin[42] = (102 << 8) + 91;
	m_objMin[43] = (102 << 8) + 79;
	m_objMin[44] = (102 << 8) + 102;
	m_objMin[45] = (102 << 8) + 109;
	m_objMin[46] = (102 << 8) + 125;
	m_objMin[47] = (102 << 8) + 7;
	m_objMin[48] = (102 << 8) + 127;
	m_objMin[49] = (102 << 8) + 111;
	m_objMin[50] = (109 << 8) + 63;
	m_objMin[51] = (109 << 8) + 6;
	m_objMin[52] = (109 << 8) + 91;
	m_objMin[53] = (109 << 8) + 79;
	m_objMin[54] = (109 << 8) + 102;
	m_objMin[55] = (109 << 8) + 109;
	m_objMin[56] = (109 << 8) + 125;
	m_objMin[57] = (109 << 8) + 7;
	m_objMin[58] = (109 << 8) + 127;
	m_objMin[59] = (109 << 8) + 111;
}


void CClock::DefineWndRegion(BOOL bRedraw)
{
	// get time
	time_t ltime;
	time(&ltime);
	// convert to local time
	struct tm* pTime;
	pTime = localtime(&ltime);
	if (pTime == NULL)
		return;

	if (m_nHour == pTime->tm_hour && m_nMin == pTime->tm_min)
		return;

	m_hWndRgn = CreateRectRgn(0, 0, 0, 0);

	// *** HOUR *** //
	m_nHour = pTime->tm_hour;

	WORD wTime = m_objHour[pTime->tm_hour];

	// 1st LED
	BYTE bt = (wTime >> 8) & 0xFF;

	if (bt & 64) // line 6
		AddRegion(0, 6);

	if (bt & 32) // line 5
		AddRegion(0, 5);

	if (bt & 16) // line 4
		AddRegion(0, 4);

	if (bt & 8) // line 3
		AddRegion(0, 3);

	if (bt & 4) // line 2
		AddRegion(0, 2);

	if (bt & 2) // line 1
		AddRegion(0, 1);

	if (bt & 1) // line 0
		AddRegion(0, 0);

	// 2nd LED
	bt = wTime & 0xFF;
	if (bt & 64) // line 6
		AddRegion(1, 6);

	if (bt & 32) // line 5
		AddRegion(1, 5);

	if (bt & 16) // line 4
		AddRegion(1, 4);

	if (bt & 8) // line 3
		AddRegion(1, 3);

	if (bt & 4) // line 2
		AddRegion(1, 2);

	if (bt & 2) // line 1
		AddRegion(1, 1);

	if (bt & 1) // line 0
		AddRegion(1, 0);

	// *** MINUTE *** //
	m_nMin = pTime->tm_min;

	wTime = m_objMin[pTime->tm_min];

	// 3rd LED
	bt = (wTime >> 8) & 0xFF;
	if (bt & 64) // line 6
		AddRegion(2, 6);

	if (bt & 32) // line 5
		AddRegion(2, 5);

	if (bt & 16) // line 4
		AddRegion(2, 4);

	if (bt & 8) // line 3
		AddRegion(2, 3);

	if (bt & 4) // line 2
		AddRegion(2, 2);

	if (bt & 2) // line 1
		AddRegion(2, 1);

	if (bt & 1) // line 0
		AddRegion(2, 0);

	// 4th LED
	bt = wTime & 0xFF;
	if (bt & 64) // line 6
		AddRegion(3, 6);

	if (bt & 32) // line 5
		AddRegion(3, 5);

	if (bt & 16) // line 4
		AddRegion(3, 4);

	if (bt & 8) // line 3
		AddRegion(3, 3);

	if (bt & 4) // line 2
		AddRegion(3, 2);

	if (bt & 2) // line 1
		AddRegion(3, 1);

	if (bt & 1) // line 0
		AddRegion(3, 0);

	AddDotRegio(55, 14);
	AddDotRegio(55, 24);

	SetWindowRgn(m_hWndRgn, bRedraw);
}

void CClock::DrawDot(HDC hdc, int xPos, int yPos)
{
	HPEN hOldPen = (HPEN)SelectObject(hdc, m_bkDotPen);

	MoveToEx(hdc, xPos + 1, yPos, NULL);
	LineTo(hdc, xPos + 5, yPos);
	MoveToEx(hdc, xPos, yPos + 1, NULL);
	LineTo(hdc, xPos + 6, yPos + 1);
	MoveToEx(hdc, xPos, yPos + 2, NULL);
	LineTo(hdc, xPos + 6, yPos + 2);
	MoveToEx(hdc, xPos, yPos + 3, NULL);
	LineTo(hdc, xPos + 6, yPos + 3);
	MoveToEx(hdc, xPos + 1, yPos + 4, NULL);
	LineTo(hdc, xPos + 5, yPos + 4);

	SelectObject(hdc, hOldPen);
}

void CClock::AddDotRegio(int xPos, int yPos)
{
	int nRet = 0;
	HRGN hObjRgn = NULL;
	HRGN hTmpRgn = NULL;

	hObjRgn = CreateRectRgn(xPos + 1, yPos, xPos + 5, yPos + 1);
	hTmpRgn = CreateRectRgn(xPos, yPos + 1, xPos + 6, yPos + 4);
	nRet = CombineRgn(hObjRgn, hObjRgn, hTmpRgn, RGN_OR);
	DeleteObject(hTmpRgn);
	hTmpRgn = CreateRectRgn(xPos + 1, yPos + 4, xPos + 5, yPos + 5);
	nRet = CombineRgn(hObjRgn, hObjRgn, hTmpRgn, RGN_OR);
	DeleteObject(hTmpRgn);

	if (hObjRgn)
	{
		CombineRgn(m_hWndRgn, m_hWndRgn, hObjRgn, RGN_OR);
		DeleteObject(hObjRgn);
	}
}

void CClock::AddRegion(int idx1, int idx2)
{
	int nRet = 0;
	HRGN hObjRgn = NULL;
	HRGN hTmpRgn = NULL;

	switch (m_obj[idx1][idx2].cType)
	{
	case 1:
		// horizontal line
		hObjRgn = CreateRectRgn(m_obj[idx1][idx2].xPos, m_obj[idx1][idx2].yPos + 2, m_obj[idx1][idx2].xPos + 18, m_obj[idx1][idx2].yPos + 3);
		hTmpRgn = CreateRectRgn(m_obj[idx1][idx2].xPos + 1, m_obj[idx1][idx2].yPos + 1, m_obj[idx1][idx2].xPos + 17, m_obj[idx1][idx2].yPos + 2);
		nRet = CombineRgn(hObjRgn, hObjRgn, hTmpRgn, RGN_OR);
		DeleteObject(hTmpRgn);
		hTmpRgn = CreateRectRgn(m_obj[idx1][idx2].xPos + 1, m_obj[idx1][idx2].yPos + 3, m_obj[idx1][idx2].xPos + 17, m_obj[idx1][idx2].yPos + 4);
		nRet = CombineRgn(hObjRgn, hObjRgn, hTmpRgn, RGN_OR);
		DeleteObject(hTmpRgn);
		hTmpRgn = CreateRectRgn(m_obj[idx1][idx2].xPos + 2, m_obj[idx1][idx2].yPos, m_obj[idx1][idx2].xPos + 16, m_obj[idx1][idx2].yPos + 1);
		nRet = CombineRgn(hObjRgn, hObjRgn, hTmpRgn, RGN_OR);
		DeleteObject(hTmpRgn);
		hTmpRgn = CreateRectRgn(m_obj[idx1][idx2].xPos + 2, m_obj[idx1][idx2].yPos + 4, m_obj[idx1][idx2].xPos + 16, m_obj[idx1][idx2].yPos + 5);
		nRet = CombineRgn(hObjRgn, hObjRgn, hTmpRgn, RGN_OR);
		DeleteObject(hTmpRgn);
		break;

	case 2:
		// vertical line
		hObjRgn = CreateRectRgn(m_obj[idx1][idx2].xPos + 2, m_obj[idx1][idx2].yPos, m_obj[idx1][idx2].xPos + 3, m_obj[idx1][idx2].yPos + 18);
		hTmpRgn = CreateRectRgn(m_obj[idx1][idx2].xPos + 1, m_obj[idx1][idx2].yPos + 1, m_obj[idx1][idx2].xPos + 2, m_obj[idx1][idx2].yPos + 17);
		nRet = CombineRgn(hObjRgn, hObjRgn, hTmpRgn, RGN_OR);
		DeleteObject(hTmpRgn);
		hTmpRgn = CreateRectRgn(m_obj[idx1][idx2].xPos + 3, m_obj[idx1][idx2].yPos + 1, m_obj[idx1][idx2].xPos + 4, m_obj[idx1][idx2].yPos + 17);
		nRet = CombineRgn(hObjRgn, hObjRgn, hTmpRgn, RGN_OR);
		DeleteObject(hTmpRgn);
		hTmpRgn = CreateRectRgn(m_obj[idx1][idx2].xPos, m_obj[idx1][idx2].yPos + 2, m_obj[idx1][idx2].xPos + 1, m_obj[idx1][idx2].yPos + 16);
		nRet = CombineRgn(hObjRgn, hObjRgn, hTmpRgn, RGN_OR);
		DeleteObject(hTmpRgn);
		hTmpRgn = CreateRectRgn(m_obj[idx1][idx2].xPos + 4, m_obj[idx1][idx2].yPos + 2, m_obj[idx1][idx2].xPos + 5, m_obj[idx1][idx2].yPos + 16);
		nRet = CombineRgn(hObjRgn, hObjRgn, hTmpRgn, RGN_OR);
		DeleteObject(hTmpRgn);
		break;
	}

	if (hObjRgn)
	{
		CombineRgn(m_hWndRgn, m_hWndRgn, hObjRgn, RGN_OR);
		DeleteObject(hObjRgn);
	}
}

void CClock::DrawObj(HDC hdc, int idx1, int idx2)
{
	HPEN hOldPen = (HPEN)SelectObject(hdc, m_bkPen);

	switch (m_obj[idx1][idx2].cType)
	{
	case 1:
		MoveToEx(hdc, m_obj[idx1][idx2].xPos, m_obj[idx1][idx2].yPos + 2, NULL);
		LineTo(hdc, m_obj[idx1][idx2].xPos + 18, m_obj[idx1][idx2].yPos + 2);
		MoveToEx(hdc, m_obj[idx1][idx2].xPos + 1, m_obj[idx1][idx2].yPos + 1, NULL);
		LineTo(hdc, m_obj[idx1][idx2].xPos + 17, m_obj[idx1][idx2].yPos + 1);
		MoveToEx(hdc, m_obj[idx1][idx2].xPos + 1, m_obj[idx1][idx2].yPos + 3, NULL);
		LineTo(hdc, m_obj[idx1][idx2].xPos + 17, m_obj[idx1][idx2].yPos + 3);
		MoveToEx(hdc, m_obj[idx1][idx2].xPos + 2, m_obj[idx1][idx2].yPos, NULL);
		LineTo(hdc, m_obj[idx1][idx2].xPos + 16, m_obj[idx1][idx2].yPos);
		MoveToEx(hdc, m_obj[idx1][idx2].xPos + 2, m_obj[idx1][idx2].yPos + 4, NULL);
		LineTo(hdc, m_obj[idx1][idx2].xPos + 16, m_obj[idx1][idx2].yPos + 4);
		break;

	case 2:
		MoveToEx(hdc, m_obj[idx1][idx2].xPos + 2, m_obj[idx1][idx2].yPos, NULL);
		LineTo(hdc, m_obj[idx1][idx2].xPos + 2, m_obj[idx1][idx2].yPos + 18);
		MoveToEx(hdc, m_obj[idx1][idx2].xPos + 1, m_obj[idx1][idx2].yPos + 1, NULL);
		LineTo(hdc, m_obj[idx1][idx2].xPos + 1, m_obj[idx1][idx2].yPos + 17);
		MoveToEx(hdc, m_obj[idx1][idx2].xPos + 3, m_obj[idx1][idx2].yPos + 1, NULL);
		LineTo(hdc, m_obj[idx1][idx2].xPos + 3, m_obj[idx1][idx2].yPos + 17);
		MoveToEx(hdc, m_obj[idx1][idx2].xPos, m_obj[idx1][idx2].yPos + 2, NULL);
		LineTo(hdc, m_obj[idx1][idx2].xPos, m_obj[idx1][idx2].yPos + 16);
		MoveToEx(hdc, m_obj[idx1][idx2].xPos + 4, m_obj[idx1][idx2].yPos + 2, NULL);
		LineTo(hdc, m_obj[idx1][idx2].xPos + 4, m_obj[idx1][idx2].yPos + 16);
		break;
	}

	SelectObject(hdc, hOldPen);
}

void CClock::DrawMin(HDC hdc, WORD wHour)
{
	BYTE bt = (wHour >> 8) & 0xFF;
	if (bt & 64) // led 7
		DrawObj(hdc, 2, 6);
	if (bt & 32) // led 6
		DrawObj(hdc, 2, 5);
	if (bt & 16) // led 5
		DrawObj(hdc, 2, 4);
	if (bt & 8) // led 4
		DrawObj(hdc, 2, 3);
	if (bt & 4) // led 3
		DrawObj(hdc, 2, 2);
	if (bt & 2) // led 2
		DrawObj(hdc, 2, 1);
	if (bt & 1) // led 1
		DrawObj(hdc, 2, 0);

	bt = wHour & 0xFF;
	if (bt & 64) // led 7
		DrawObj(hdc, 3, 6);
	if (bt & 32) // led 6
		DrawObj(hdc, 3, 5);
	if (bt & 16) // led 5
		DrawObj(hdc, 3, 4);
	if (bt & 8) // led 4
		DrawObj(hdc, 3, 3);
	if (bt & 4) // led 3
		DrawObj(hdc, 3, 2);
	if (bt & 2) // led 2
		DrawObj(hdc, 3, 1);
	if (bt & 1) // led 1
		DrawObj(hdc, 3, 0);
}

void CClock::DrawHour(HDC hdc, WORD wHour)
{
	BYTE bt = (wHour >> 8) & 0xFF;
	if (bt & 64) // led 7
		DrawObj(hdc, 0, 6);
	if (bt & 32) // led 6
		DrawObj(hdc, 0, 5);
	if (bt & 16) // led 5
		DrawObj(hdc, 0, 4);
	if (bt & 8) // led 4
		DrawObj(hdc, 0, 3);
	if (bt & 4) // led 3
		DrawObj(hdc, 0, 2);
	if (bt & 2) // led 2
		DrawObj(hdc, 0, 1);
	if (bt & 1) // led 1
		DrawObj(hdc, 0, 0);

	bt = wHour & 0xFF;
	if (bt & 64) // led 7
		DrawObj(hdc, 1, 6);
	if (bt & 32) // led 6
		DrawObj(hdc, 1, 5);
	if (bt & 16) // led 5
		DrawObj(hdc, 1, 4);
	if (bt & 8) // led 4
		DrawObj(hdc, 1, 3);
	if (bt & 4) // led 3
		DrawObj(hdc, 1, 2);
	if (bt & 2) // led 2
		DrawObj(hdc, 1, 1);
	if (bt & 1) // led 1
		DrawObj(hdc, 1, 0);
}

void CClock::DrawTime(HDC hdc, int nHour, int nMin)
{
	if (nHour < 0 || nHour > 23 || nMin < 0 || nMin > 59)
		return;

	DrawHour(hdc, m_objHour[nHour]);
	DrawDot(hdc, 55, 14);
	DrawDot(hdc, 55, 24);
	DrawMin(hdc, m_objMin[nMin]);
}

void CClock::OnPaint()
{
	PAINTSTRUCT ps;
	HDC hdc = ::BeginPaint(this->m_hWnd, &ps);
	//	HDC hdc = ::GetDC(this->m_hWnd);
	DrawTime(hdc, m_nHour, m_nMin);
	//	::ReleaseDC(this->m_hWnd, hdc);
	::EndPaint(this->m_hWnd, &ps);
}

void CClock::OnTimeTick()
{
	// *** set window region from time *** //
	DefineWndRegion(TRUE);

	HDC hdc = ::GetDC(this->m_hWnd);
	if (hdc)
	{
		if (m_objColor != CLOCK_COLOR1)
		{
			m_objColor = CLOCK_COLOR1;
			if (m_bkPen)
				DeleteObject(m_bkPen);
			m_bkPen = CreatePen(PS_SOLID, 1, m_objColor);

			DrawHour(hdc, m_objHour[m_nHour]);
			DrawMin(hdc, m_objMin[m_nMin]);
		}

		if (m_dotColor == CLOCK_COLOR1)
			m_dotColor = CLOCK_COLOR2;
		else
			m_dotColor = CLOCK_COLOR1;
		if (m_bkDotPen)
			DeleteObject(m_bkDotPen);
		m_bkDotPen = CreatePen(PS_SOLID, 1, m_dotColor);
		DrawDot(hdc, 55, 14);
		DrawDot(hdc, 55, 24);
	}
	::ReleaseDC(this->m_hWnd, hdc);
}
UINT CClock::OnNcHitTest(CPoint point)
{
	return HTCAPTION;
}

void CClock::OnDestroy()
{
	if (m_bkPen)
		DeleteObject(m_bkPen);
	if (m_hWndRgn)
		DeleteObject(m_hWndRgn);
}
